// lzw_main.h: test for the lzw::compression class.
//
//////////////////////////////////////////////////////////////////////


#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <iostream>

#define BLOCK_SIZE (16*1024)
#define MiB (1024*1024)

#include "lzw/lzw_compression.h"

#include "utils/string_utils.h"


int main (int narg, char** argvp)
{
    if (narg < 2)
    {
        std::cout << "The name of the input file is required." << std::endl;
        return 1;
    }

    ssize_t max_read(std::numeric_limits<ssize_t>::max());

    if (narg > 2)
    {
        max_read = atoi(argvp[2]) * MiB;
    }

    ssize_t heap_size_in_k(32*1024);

    if (narg > 3)
    {
        heap_size_in_k = atoi(argvp[3]);
    }

    lzw::compression    compression(heap_size_in_k * 1024);

    // open the file
    int file(open(argvp[1], O_RDONLY));

    if (!file)
    {
        std::cout << "failed to open " << argvp[1] << std::endl;
        return 2;
    }

    // read in blo  cks
    char        buf[BLOCK_SIZE];
    ssize_t     read_now;
    ssize_t     read_total(0);
    ssize_t     comp_total(0);
    ssize_t     MiBMask(ssize_t(MiB)-1);

    while ((read_now = read(file, buf, BLOCK_SIZE)) > 0)
    {
        read_total += read_now;
        comp_total += compression.compress(buf, read_now, buf, BLOCK_SIZE);

        if ((read_total & MiBMask) == 0)
        {
            std::cout << "read: " << read_total/MiB << " MiB, compressed to: " << comp_total/MiB << " MiB." << std::endl;

            if (read_total >= max_read) break;
        }
    }

    close(file);

    if (strcmp(argvp[narg-1], "metrics") == 0) compression.metrics();

    return 0;
}
